@@ -9,7 +9,7 @@
 package XML::LibXML::Error;
 
 use strict;
-use vars qw($AUTOLOAD @error_domains $VERSION $WARNINGS);
+use vars qw(@error_domains $VERSION $WARNINGS);
 use Carp;
 use overload 
   '""' => \&as_string,
@@ -22,7 +22,7 @@ use overload
   fallback => 1;
 
 $WARNINGS = 0; # 0: supress, 1: report via warn, 2: report via die
-$VERSION = "1.70"; # VERSION TEMPLATE: DO NOT CHANGE
+$VERSION = "1.71"; # VERSION TEMPLATE: DO NOT CHANGE
 
 use constant XML_ERR_NONE	     => 0;
 use constant XML_ERR_WARNING	     => 1; # A simple warning
@@ -52,14 +52,28 @@ use constant XML_ERR_FROM_RELAXNGV   => 19; # The Relax-NG validator module
 use constant XML_ERR_FROM_CATALOG    => 20; # The Catalog module
 use constant XML_ERR_FROM_C14N	     => 21; # The Canonicalization module
 use constant XML_ERR_FROM_XSLT	     => 22; # The XSLT engine from libxslt
-use constant XML_ERR_FROM_VALID	     => 23; # The validaton module
+use constant XML_ERR_FROM_VALID	     => 23; # The DTD validation module with valid context
+use constant XML_ERR_FROM_CHECK	     => 24; # The error-checking module
+use constant XML_ERR_FROM_WRITER     => 25; # The xmlwriter module
+use constant XML_ERR_FROM_MODULE     => 26; # The dynamically-loaded module module
+use constant XML_ERR_FROM_I18N	     => 27; # The module handling character conversion
+use constant XML_ERR_FROM_SCHEMATRONV=> 28; # The Schematron validator module
 
 @error_domains = ("", "parser", "tree", "namespace", "validity",
 		  "HTML parser", "memory", "output", "I/O", "ftp",
 		  "http", "XInclude", "XPath", "xpointer", "regexp",
 		  "Schemas datatype", "Schemas parser", "Schemas validity", 
 		  "Relax-NG parser", "Relax-NG validity",
-		  "Catalog", "C14N", "XSLT", "validity");
+		  "Catalog", "C14N", "XSLT", "validity", "error-checking",
+		  "xmlwriter", "dynamic loading", "i18n",
+		  "Schematron validity");
+
+for my $field (qw<code _prev level file line nodename message column context
+                  str1 str2 str3 num1 num2>) {
+    my $method = sub { $_[0]{$field} };
+    no strict 'refs';
+    *$field = $method;
+}
 
 { 
 
@@ -149,28 +163,17 @@ use constant XML_ERR_FROM_VALID	     => 23; # The validaton module
 }
 
 
-sub AUTOLOAD {
-  my $self=shift;
-  return undef unless ref($self);
-  my $sub = $AUTOLOAD;
-  $sub =~ s/.*:://;
-  if ($sub=~/^(?:code|_prev|level|file|line|domain|nodename|message|column|context|str[123]|num[12])$/) {
-    return $self->{$sub};
-  } else {
-    croak("Unknown error field $sub");
-  }
-}
-
 # backward compatibility
 sub int1 { $_[0]->num1 }
 sub int2 { $_[0]->num2 }
 
-sub DESTROY {}
-
 sub domain {
     my ($self)=@_;
     return undef unless ref($self);
-    return $error_domains[$self->{domain}];
+    my $domain = $self->{domain};
+    # Newer versions of libxml2 might yield errors in domains that aren't
+    # listed above.  Invent something reasonable in that case.
+    return $domain < @error_domains ? $error_domains[$domain] : "domain_$domain";
 }
 
 sub as_string {
@@ -202,7 +205,7 @@ sub as_string {
         $where.=": element ".$self->{nodename};
     }
     $msg.=$where.": " if $where ne "";
-    $msg.=$error_domains[$self->{domain}]." ".$level." :";
+    $msg.=$self->domain." ".$level." :";
     my $str=$self->{message}||"";
     chomp($str);
     $msg.=" ".$str."\n";
@@ -211,6 +214,12 @@ sub as_string {
       $msg.=$self->{str1}."\n";
       $msg.=(" " x $self->{num1})."^\n";
     } elsif (defined $self->{context}) {
+      # If the error relates to character-encoding problems in the context,
+      # then doing textual operations on it will spew warnings that
+      # XML::LibXML can do nothing to fix.  So just disable all such
+      # warnings.  This has the pleasing benefit of making the test suite
+      # run warning-free.
+      no warnings 'utf8';
       my $context = $self->{context};
       $msg.=$context."\n";
       $context = substr($context,0,$self->{column});
